#-------------------------------------------------------------------------------
## LOAD PACKAGES, SET WORKING DIRECTORY
library(jagsUI)
## SET WORKING DIRECTORY
setwd("C:/.../")   ## CHANGE AS REQUIRED
################################################################################
##  INTEGRATED MODEL FOR ANTARCTIC FUR SEAL MALES
#-------------------------------------------------------------------------------
##
#   JAGS MODEL
sink("AFSMjsms.jags")
cat("
  model{
    ### PRIORS
    # RECRUITMENT PROBABILITY (mature seals = m)
    for(i in 1:6){
      m[i] <- 0
    }
    for(i in 1:5){
      mm[i] ~ dbeta(aBeta[i], bBeta[i])   # priors parameters for beta distributions
      m[i + 6] <- mm[i]
    }
    ## INITIAL POPULATION SIZES
    for(i in 1:Nages){
      n[1, i] ~ dnorm(N1m[i], 0.001)T(0, )
      N[1, i] <- round(n[1, i])
    }
    ## SEX RATIO
    for(i in 1:nyears){
      sr[i] <- srp[i] / np[i]
      srp[i] ~ dbin(pM[i], np[i])
    }
    ## PUP AND JUVENILE SURVIVAL PRIORS
    for(t in 1:nyears){
      phi0[t] ~ dbeta(alphaS[t], betaS[t])
      phi1[t] ~ dbeta(267.107,233.487)
      # phij[t] ~ dbeta(1075.389,790.8746)
      for(age in 1:Nages){
        phij[t, age] ~ dbeta(surAlpha[age],surBeta[age])
      }
    }
    ## ESTIMATED POPULATION SIZE OF PRETERRITORIALS AND NEW RECRUITS TOGETHER
    for(t in 1:nyears){
      Nall[t] <- sum(N[t, ])              #### OLD Nestim[t]
      NMall[t] <- inprod(N[t, ], m[])     #### OLD Nestim2[t]
    }
    # Vectors of population size classes by age 
    for(t in 1:nyears){
      for(age in 1:Nages){
        NM[t, age] <- N[t, age] * m[age]     # NM are maturing individuals
      }
    }
    for(t in 1:nyears){
      for(age in 1:Nages){
        NP[t, age] <- N[t, age] - NM[t, age]  # NP are non-maturing individuals
      }
    }
    for(t in 1:nyears){
      NPall[t] <- sum(NP[t, ])
    } 
    #
    ### Likelihoods
    ##  Population count data (state-space model)
    #   System process
    for(t in 2:nyears){
      mean1[t] <- sr[t] * np[t] * phi0[t] * phi1[t]
      N[t, 1] ~ dpois(mean1[t])
      for(age in 2:Nages){
        meanN[t, age] <- phij[t - 1, age - 1] * N[t - 1, age - 1]
        N[t, age] ~ dpois(meanN[t, age])
      }
    }
    #
    ## JOLLY-SEBER MULTISTATE MODEL
    # Priors
    for(t in 1:(nyears - 1)){
      logit(phiT[t]) <- muT + epsilonT[t]       # temporal random efects on survival of territorials
      epsilonT[t] ~ dnorm(0, tauT)T(-15,15)
      phiN[t] <- mean.phiN                      # constant survival of N
      psiTN[t] <- mean.psiTN                    # Constant mean.psiNT
      psiNT[t] <- mean.psiNT                    # Constant mean.psiNT
      gamma[t] ~ dunif(0, 1)                    # Prior for return
      logit(p[t]) <- muP + epsilonP[t]          # temporal random efects on recapture of territorials
      epsilonP[t] ~ dnorm(0, tauP)T(-15,15)
    }
    #
    ## survival and psiTN/NT
    muT <- log(mean.phiT / (1 - mean.phiT))
    mean.phiT ~ dunif(0, 1)                     # Prior for mean survival of B
    tauT <- pow(sigmaT, -2)
    sigmaT ~ dunif(0, 10)                       # Prior on standard deviation
    sigma2T <- pow(sigmaT, 2)                   # Temporal variance
    #
    mean.phiN ~ dunif(0, 1)  
    #
    mean.psiTN ~ dunif(0, 1)                    # Prior for mean temporary emigration
    #
    mean.psiNT ~ dunif(0, 1)                    # Prior for mean temporary immigration
    #
    muP <- log(mean.p / (1 - mean.p))         # Logit transformation
    mean.p ~ dunif(0, 1)                      # Prior for mean recapture
    tauP <- pow(sigmaP, -2)
    sigmaP ~ dunif(0, 5)                      # Prior for standard deviation
    sigma2P <- pow(sigmaP, 2)
    #
    ## transition and observation matrices   
    for (i in 1:M){  
      # probabilities of state S(t+1) given S(t)
      for (t in 1:(nyears-1)){
        ps[1, i, t, 1] <- 1 - gamma[t]
        ps[1, i, t, 2] <- gamma[t]
        ps[1, i, t, 3] <- 0
        ps[1, i, t, 4] <- 0
        ps[2, i, t, 1] <- 0
        ps[2, i, t, 2] <- phiT[t] * (1 - psiTN[t])
        ps[2, i, t, 3] <- phiT[t] * psiTN[t]
        ps[2, i, t, 4] <- 1 - phiT[t]
        ps[3, i, t, 1] <- 0
        ps[3, i, t, 2] <- phiN[t] * psiNT[t]
        ps[3, i, t, 3] <- phiN[t] * (1 - psiNT[t])
        ps[3, i, t, 4] <- 1 - phiN[t]
        ps[4, i, t, 1] <- 0
        ps[4, i, t, 2] <- 0
        ps[4, i, t, 3] <- 0
        ps[4, i, t, 4] <- 1
        #
        ## probabilities of O(t) given S(t)
        po[1, i, t, 1] <- 0
        po[1, i, t, 2] <- 1
        po[2, i, t, 1] <- p[t]
        po[2, i, t, 2] <- 1 - p[t]
        po[3, i, t, 1] <- 0
        po[3, i, t, 2] <- 1
        po[4, i, t, 1] <- 0
        po[4, i, t, 2] <- 1
      }
    }
    ## Likelihood 
    for(i in 1:M){
      # latent state at first occasion
      z[i, 1] <- 1   # all M seals are in state 1 (pre-breeder) at t = 1
      for(t in 2:nyears){
        # State process: draw S(t) given S(t-1)
        z[i, t] ~ dcat(ps[z[i, t - 1], i, t - 1, ])
        # Observation process: draw O(t) given S(t)
        y[i, t] ~ dcat(po[z[i, t], i, t - 1, ])
      }
    }
    # Derived population parameters
    for(t in 1:(nyears - 1)){
      qgamma[t] <- 1 - gamma[t]
    }
    cprob[1] <- gamma[1]
    for(t in 2:(nyears - 1)){
      cprob[t] <- gamma[t] * prod(qgamma[1:(t - 1)])
    }
    psi <- sum(cprob[])                          # Inclusion probability
    for(t in 1:(nyears - 1)){
      b[t] <- cprob[t] / psi                     # Entry probability
    }
    for(i in 1:M){
      for(t in 2:nyears){
        alt[i, t - 1] <- equals(z[i, t], 2)
        aln[i, t - 1] <- equals(z[i, t], 3)
      }
      for(t in 1:(nyears - 1)){
        d[i, t] <- equals(z[i, t] - (alt[i, t] + aln[i, t]), 0)
      }
      alive[i] <- sum(alt[i, ] + aln[i, ])
    }
    for(t in 1:(nyears - 1)){
      NT[t] <- sum(alt[, t])                          # Actual population size of territorials
      NN[t] <- sum(aln[, t])                          # Actual population size of non-territorials
      NATot[t] <- NT[t] + NN[t]                       # Actual mature population size
      B[t] <- sum(d[, t])                             # Number of entries - recruitment and immigration-
    }
    for (i in 1:M){
      w[i] <- 1 - equals(alive[i], 0)
    }
    Nsuper <- sum(w[])                                # Superpopulation size
    ## END OF JOLLY-SEBER
    ##
    ## DERIVED PARAMETERS
    for(t in 1:(nyears - 1)){
      NI[t] <- B[t] - NMall[t]                        # immigrants
      Ntot[t] <- NATot[t] + NPall[t]                  # total population size
    }
    for(t in 1:(nyears - 2)){
      lambda[t] <- Ntot[t+1] / Ntot[t]                # population growth rate
      loglambda[t] <- log(lambda[t])                  # log-lambda
    }
    meanlambda <- exp((1/(nyears-2))*sum(loglambda[1:(nyears-2)]))  # geometric mean
    #
    ## Poisson GAMM for daily counts
    eta <- X %*% bb ## linear predictor
    for(i in 1:nn){
      mu[i] <-  exp(eta[i] + eps[i])
      eps[i] ~ dnorm(0, tau)
    } ## expected response
    for(i in 1:nn){
      yy[i] ~ dpois(mu[i])
    } ## response
    ## random effects priors
    tau <- pow(sd, -2)
    sd ~ dunif(0, 5)
    ## Parametric effect priors CHECK tau=1/34^2 is appropriate!
    for(i in 1:1){
      bb[i] ~ dnorm(0,0.00068)
    }
    ## prior for s(x0)... 
    for(i in c(2:9)){
      bb[i] ~ dnorm(0, lambda2[1])
    }
    for(i in c(10)){
      bb[i] ~ dnorm(0, lambda2[2])
    }
    ## smoothing parameter priors CHECK...
    for(i in 1:2){
      lambda2[i] ~ dgamma(.05,.005)
      rho[i] <- log(lambda2[i])
    }
    ## OTHER DERIVED QUANTITIES
    for(j in 1:7){
      NATot.nc[j] <- NATot[12] / mu[32+j]
      NT.nc[j] <- NT[12] / mu[32+j]
    }
  }
  ",fill = TRUE)
sink()
#
##
#  DATA
## READ DATA FILES
N1m <- readRDS("N1m.Rds")
aBeta <- readRDS("aBeta.Rds")
bBeta <- readRDS("bBeta.Rds")
pMal <- readRDS("pMal.Rds")
np <- saveRDS("np.Rds")
alphaS <- readRDS("alphaS.Rds")
betaS <- readRDS("betaS.Rds")
surAlpha <- readRDS("surAlpha.Rds")
surBeta <- readRDS("surBeta.Rds")
CH.ms <- readRDS("CH.ms.Rds")
yy <- readRDS("yy.Rds")
nn <- readRDS("nn.Rds")
X <- readRDS("X.Rds")
##
##
jags.data <- list(Nages = 11, N1m = N1m, aBeta = aBeta, bBeta = bBeta, pM = pMal[1:14], np = prod9522[1:14], 
                  alphaS = STTbeta$alpha, betaS = STTbeta$beta, surAlpha = surBetas$alpha, surBeta = surBetas$beta,
                  y = CH.ms, nyears = dim(CH.ms)[2], M = dim(CH.ms)[1], yy = jd$jags.data$y, nn = jd$jags.data$n, X = jd$jags.data$X)
#
## INITIAL VALUES
###  Initial values need to be specified for the latent state z must correspond to the true state; not the same as the observed state.
###  "1" for all latent states before an individual is observed
###  "2" when the individual was observed alive and nesting 
###  "3" between observations
###  "4" after the last observation.
##
# code to generate such values after data augmentation
js.ms.init <- function(ch, nz){
  # define states
  state <- ch
  state[state == 2] <- NA
  # "2" individual observed alive and nesting 
  state[state == 1] <- 2
  for(i in 1:(dim(state)[1] - nz)){
    n1 <- min(which(state[i, ] == 2))
    n2 <- max(which(state[i, ] == 2))
    n3 <- dim(state)[2]
    if(n2 > n1)
      state[i, (n1 + 1):n2] <- ifelse(is.na(state[i, (n1 + 1):n2]), 3, 2)
    state[i, 1:(n1 - 1)] <- 1
    if(n3 > n2)
      state[i, (n2 + 1):n3] <- 4  
  }
  state[which(is.na(state))] <- 1
  state[, 1] <- NA
  return(state)
}
#
## initial values
inits <- function(){
  list(mean.phiT = runif(1, 0, 1),
       mean.phiN = runif(1, 0, 1),
       mean.psiTN = runif(1, 0, 1), 
       mean.psiNT = runif(1, 0, 1), 
       mean.p = runif(1, 0, 1), 
       sigmaT = runif(1, 0, 1),
       sigmaP = runif(1, 0, 1),
       z = js.ms.init(CH.ms, nz),
       bb = jd$jags.ini$b,
       lambda2 = jd$jags.ini$lambda
  )
}    
#
## MCMC settings
ni <- 50000 
nt <- 6
nb <- 20000
nc <- 3
#
## Parameters monitored
params <- c("phij", "phiT", "mean.phiT", "phiN", "mean.phiN", "psiTN", "mean.psiTN", "psiNT", "mean.psiNT", "p", "mean.p",
            "sigma2T", "sigma2P", "m", "b", "Nsuper", "NT", "NN", "NATot", "B", "NM", "NP", "NMall", "NPall", "Ntot", "NI", 
            "loglambda", "meanlambda", "NATot.nc", "NT.nc", "bb", "rho", "mu", "sd", "eps")
#
sta <- Sys.time()
out <- jags(jags.data, inits, params, "AFSMjsms.jags", n.chains = nc, n.thin =  nt, n.iter = ni, n.burnin = nb) 
Sys.time() - sta
##
################################################################################